using System;
using System.Collections;
using System.Collections.Generic;
using System.Globalization;
using Android.App;
using Android.Content;
using Android.OS;
using Android.Views;
using Android.Widget;
using Master.AndroidCode.API;
using Master.Core;

namespace Master.AndroidCode.GUI
{
    [Activity(Label = "Device selected")]
    public class SelectedDeviceActivity : Activity
    {
        ArrayAdapter _listAdapter;
        bool _isBound = false;
        GuitarControllerService.GuitarControllerServiceBinder _binder;
        GuitarControllerServiceConnection _guitarControllerServiceConnection;
        Intent _guitarControllerServiceIntent;
        private IPedal _device;
        private readonly List<TableRow> _tableRows = new List<TableRow>();

        protected override void OnCreate(Bundle bundle)
        {
            base.OnCreate(bundle);
           
            SetContentView(Resource.Layout.SelectedDevice);
            _guitarControllerServiceIntent = new Intent("com.aalbergAudio.GuitarController");

        }

        private void Save()
        {
            var alert = new AlertDialog.Builder(this);
            alert.SetView(LayoutInflater.Inflate(Resource.Layout.SaveDialog, null));

            alert.SetPositiveButton("Save to presets",
                (sender, e) => SaveAndUpdate());
            alert.Create().Show();
            
        }

        private void SaveAndUpdate()
        {
            _device.SaveToPresets("test" + new Random().Next());
            _listAdapter = new ArrayAdapter(this, Android.Resource.Layout.SimpleListItem1,
                _device.GetPresets());
            UpdateInterface();
        }

        private void Decrease(Option option, TextView valueBox)
        {
            _device.DecreaseOption(option.Name);
            valueBox.Text = _device.Options[option.Name].Value.ToString(CultureInfo.InvariantCulture);
        }
        private void Increase(Option option, TextView valueBox)
        {
            _device.IncreaseOption(option.Name);
            valueBox.Text = _device.Options[option.Name].Value.ToString(CultureInfo.InvariantCulture);
        }

        protected override void OnStart()
        {
            base.OnStart();
            _guitarControllerServiceConnection = new GuitarControllerServiceConnection(this);
            BindService(_guitarControllerServiceIntent, _guitarControllerServiceConnection, Bind.AutoCreate);

        }

        protected void CreateInterface()
        {

            _device = _binder.GetGuitarControllerService().GetConnectedDevice(Intent.GetStringExtra("DeviceID"));

            FindViewById<TextView>(Resource.Id.name).Text = _device.Name;
            UpdateOptions();
           
        }

        private void UpdateOptions()
        {
            foreach (var option in _device.Options.Values)
            {
                var nameBox = new TextView(this) {Text = option.Name};
                var valueBox = new TextView(this) {Text = option.Value.ToString()};
                var increase = new Button(this) {Text = "Increase value"};
                var decrease = new Button(this) {Text = "Decrease value"};
                increase.Click += (sender, e) => Increase(option, valueBox);
                decrease.Click += (sender, e) => Decrease(option, valueBox);
                var tableRow = new TableRow(this);
                _tableRows.Add(tableRow);
                tableRow.AddView(nameBox);
                tableRow.AddView(valueBox);
                tableRow.AddView(increase);
                tableRow.AddView(decrease);
                FindViewById<TableLayout>(Resource.Id.table).AddView(tableRow);
            }
            var buttonRow = new TableRow(this);
            _tableRows.Add(buttonRow);
            var button = new Button(this) { Text = "Save to presets" };
            button.Click += (sender, e) => Save();
            buttonRow.AddView(button);
            FindViewById<TableLayout>(Resource.Id.table).AddView(buttonRow);
            var presets = new ListView(this);
            _listAdapter = new ArrayAdapter(this, Android.Resource.Layout.SimpleListItem1,
                _device.GetPresets());
            presets.Adapter = _listAdapter;
            var listRow = new TableRow(this);
            _tableRows.Add(listRow);
            listRow.AddView(presets);
            FindViewById<TableLayout>(Resource.Id.table).AddView(listRow);
            presets.ItemClick += OnListItemClick;
        }

        protected void OnListItemClick(object sender, Android.Widget.AdapterView.ItemClickEventArgs e)
        {
            var presetName = _device.GetPresets()[e.Position].GetId();
            Console.WriteLine(presetName);
            _device.ApplyPreset(presetName);
            UpdateInterface();
        }

        private void UpdateInterface()
        {

            foreach (var tableRow in _tableRows)
            {
                FindViewById<TableLayout>(Resource.Id.table).RemoveView(tableRow);
            }
            UpdateOptions();
        }
        protected override void OnStop()
        {
            base.OnStop();
            if (_isBound)
            {
                UnbindService(_guitarControllerServiceConnection);
                _isBound = false;
            }
        }

        class GuitarControllerServiceConnection : Java.Lang.Object, IServiceConnection
        {
            private readonly SelectedDeviceActivity _activity;

            public GuitarControllerServiceConnection(SelectedDeviceActivity activity)
            {
                _activity = activity;
            }

            public void OnServiceConnected(ComponentName name, IBinder service)
            {
                var guitarControllerServiceBinder = service as GuitarControllerService.GuitarControllerServiceBinder;
                if (guitarControllerServiceBinder != null)
                {
                    var binder = (GuitarControllerService.GuitarControllerServiceBinder)service;
                    _activity._binder = binder;
                    _activity._isBound = true;
                }
                _activity.CreateInterface();
            }

            public void OnServiceDisconnected(ComponentName name)
            {
                _activity._isBound = false;
            }
        }
    }
}