using System.Collections.Generic;
using Android.Hardware.Usb;

namespace Master.AndroidCode.Testing
{
    class MockUsbManager :ITestableUsbManager
    {
        readonly IDictionary<string, ITestableUsbDevice> _deviceList;
        public IDictionary<string, ITestableUsbDevice> DeviceList
        {
            get { return _deviceList; }
        }

        public MockUsbManager()
        {
            _deviceList = new Dictionary<string, ITestableUsbDevice>
            {
                {"GuitarController", new MockUsbDevice("GuitarController")}
            };
        }

        /// <param name="device">the device to open</param>
        /// <summary>
        /// Opens the device so it can be used to send and receive
        ///  data using <c><see cref="T:Android.Hardware.Usb.UsbRequest"/></c>.
        /// </summary>
        /// <returns>
        /// To be added.
        /// </returns>
        /// <remarks>
        /// <para tool="javadoc-to-mdoc">
        /// Opens the device so it can be used to send and receive
        ///  data using <c><see cref="T:Android.Hardware.Usb.UsbRequest"/></c>.
        /// </para>
        /// <para tool="javadoc-to-mdoc">
        /// <format type="text/html"><a href="http://developer.android.com/reference/android/hardware/usb/UsbManager.html#openDevice(android.hardware.usb.UsbDevice)" target="_blank">[Android Documentation]</a></format>
        /// </para>
        /// </remarks>
        /// <since version="Added in API level 12"/>
        public  ITestableUsbDeviceConnection OpenDevice(ITestableUsbDevice device)
        {
            return new MockUsbDeviceConnection();
        }

        public UsbAccessory[] GetAccessoryList()
        {
            return null;
        }
    }
}