using System;
using System.IO;
using System.Linq;
using System.Text;
using Android.App;
using Android.Hardware.Usb;
using Android.Util;
using Master.AndroidCode.Testing;
using Master.Core;

namespace Master.AndroidCode
{
    public class UsbController : AbstractController
    {
        public const int FirmwareEndpoint = 3;
        public const int Interface = 1;
        public const int SetOptionEndpoint = 2;
        public const int GetDevicesInterface = 1;

        private readonly ITestableUsbDevice _device;
        private readonly ITestableUsbManager _manager;


        
        public UsbController(ITestableUsbManager manager)
        {
            _manager = manager;

            var accessoryList = _manager.DeviceList;
            _device = accessoryList.FirstOrDefault(a => a.Value.DeviceName == "GuitarController").Value;

        }

        public override bool IsConnected()
        {
            return _device is MockUsbDevice;
        }

        public override void SendFirmwareToDevice(string toSend)
        {
            var buffer = Encoding.Unicode.GetBytes(toSend);
            //Should be something else?
            var interf = _device.GetInterface(Interface);
            var endpoint = interf.GetEndpoint(FirmwareEndpoint);
            if (endpoint.Type != UsbAddressing.XferBulk && endpoint.Direction != UsbAddressing.Out)
            {
                throw new IOException("Something wrong with the usbprotocol");
            }
            var connection = _manager.OpenDevice(_device);
            connection.ClaimInterface(interf, true);
            //should change?
            if (connection.BulkTransfer(endpoint, buffer, buffer.Length, 500) >= 0)
                connection.ReleaseInterface(interf);
        }


        public override string SaveName { get; set; }



        protected override string GetConnectedDeviceString()
        {
            var buffer = new byte[4096];
            var result = "";
            //Should be something else?
            var interf = _device.GetInterface(Interface);
            var endpoint = interf.GetEndpoint(GetDevicesInterface);
            if ((endpoint.Type != UsbAddressing.XferBulk && endpoint.Direction != UsbAddressing.In))
            {
                throw new IOException("Something wrong with the usbprotocol");
            }
            var connection = _manager.OpenDevice(_device);
            connection.ClaimInterface(interf, true);

            if (connection.BulkTransfer(endpoint, buffer, 4096, 500) < 0) return result;
            result = Encoding.Unicode.GetString(buffer);
            connection.ReleaseInterface(interf);
            Console.WriteLine(result);
            return result;
        }

        public override void UpdateOption(string id, string option, int value)
        {
            SendStringToDevice(id + "," + option + ":" + value);
        }

        public void SendStringToDevice(string toSend)
        {
            var buffer = new byte[toSend.Length*sizeof (char)];
            Buffer.BlockCopy(toSend.ToCharArray(), 0, buffer, 0, buffer.Length);
            //Should be something else?
            var interf = _device.GetInterface(Interface);
            var endpoint = interf.GetEndpoint(SetOptionEndpoint);
            if (endpoint.Type != UsbAddressing.XferBulk && endpoint.Direction != UsbAddressing.Out)
            {
                throw new IOException("Something wrong with the usbprotocol");
            }
            var connection = _manager.OpenDevice(_device);
            connection.ClaimInterface(interf, true);

            if (connection.BulkTransfer(endpoint, buffer, buffer.Length, 500) >= 0)
                connection.ReleaseInterface(interf);
        }
    }
}